import argparse
import asyncio
from collections import defaultdict

from .classes import get_all_hardlinks
from .classes import ShredDir
from .classes import ShredFile
from .logs import logger


async def main(job: argparse.Namespace) -> bool:
    """
    This is the main function for processing a shred request.
    It is called by the CLI and builds a job queue based on the arguments passed.
    """

    # Expand all directories and files, and collect mount point information
    tasks = []
    for path in job.paths:
        if await path.is_file():
            logger.info(f"Adding file: {path}")
            tasks.append(ShredFile(path))
        elif await path.is_dir():
            logger.info(f"Adding directory: {path}")
            tasks.append(ShredDir(path, recursive=job.recursive))
        else:
            raise TypeError(f"Not a file or directory: {path}")
    new_paths = set(await asyncio.gather(*tasks))

    # Try to delete hardlinks based on the filesystem type
    job.paths = await get_all_hardlinks(new_paths)

    # Just in case, use "find" to delete any remaining hardlinks
    # from the mount point
    logger.info("Deleting remaining hardlinks using find")
    inodes_in_mount_points = defaultdict(set)
    for path in job.paths:
        inodes_in_mount_points[path.mount_point].add(path.inode)

    for mount_point, inodes in inodes_in_mount_points.items():
        async for item in mount_point.rglob("*"):
            if item.name == "." or item.name == "..":
                continue
            if item.stat().st_ino in inodes:
                log_buf = f"Deleting hardlink: {item.path}"
                if not job.dryrun:
                    log_buf = "DRY RUN " + log_buf
                    await item.path.unlink()
                logger.info(log_buf)

    # Shred all physical files including hardlinks
    for path in job.paths:
        if isinstance(path, ShredFile):
            await path.shred(hash=job.exhaustive, dryrun=job.dryrun)
        elif isinstance(path, ShredDir):
            await path.shred(hash=job.exhaustive, dryrun=job.dryrun)

    logger.info("Done")
